/*
** Copyright (C) 1998,1999 Martin Roesch <roesch@clark.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/******************************************************************************
 *
 * Program: Snort
 *
 * Purpose: This is a fairly generic sniffing program which does some nice
 *          logging.  You can tell it what the "home" network is and it will
 *          log all traffic in terms of the remote side of the connection.
 *          The program can take BPF-style filtering commands at the command
 *          line and filter out the packets it receives to just the network
 *          of interest.  It can also do rule based logging based upon packet
 *          headers or application layer content.  UPDATE: 7/20/99 This program
 *          is almost 100% different from what the above says now 7 months 
 *          later.  Check out the README file for good info on what you can do
 *          with Snort.
 *
 * Author: Martin Roesch (roesch@clark.net)
 *
 * Last Modified: 8/06/99
 *
 * Comments: Ideas and code stolen liberally from Mike Borella's IP Grab 
 *           program. Check out his stuff at http://www.borella.net.  I
 *           also have ripped some util functions from TCPdump, plus Mike's
 *           prog is derived from it as well.  All hail TCPdump....
 *
 * HP-UX 10.x note from Chris Sylvain:
 * if you run snort and receive the error message
 *  "ERROR: OpenPcap() device lan0 open:
 *                    recv_ack: promisc_phys: Invalid argument"
 * it's because there's another program running using the DLPI service.
 * The HP-UX implementation doesn't allow more than one libpcap program
 * at a time to run, unlike Linux.
 *
 ******************************************************************************/

/*  I N C L U D E S  **********************************************************/
#include "snort.h"




/****************************************************************************
 *
 * Function: main(int, char *)
 *
 * Purpose:  Handle program entry and exit, call main prog sections
 *
 * Arguments: -d => dump the application layer data
 *            -c => use configuration file
 *            -a => display ARP packets
 *            -n => exit after receiving x packets
 *            -i => listen on interface x
 *            -l => log to directory x
 *            -h => set "home" network to x
 *            -v => be verbose
 *            -V => show the version number and exit
 *            -x => display IPX packets
 *            -D => run Snort in the background
 *            -? => show the program help data and exit
 *            list => a list of BPF commands can also be sent to the program
 *                    to control what data gets captured
 *
 * Returns: 0 => normal exit, 1 => exit on error
 *
 ****************************************************************************/
int main(int argc, char *argv[])
{
   /* make this prog behave nicely when signals come along */
   signal(SIGKILL, CleanExit);
   signal(SIGTERM, CleanExit);
   signal(SIGINT, CleanExit);
   signal(SIGQUIT, CleanExit);
   signal(SIGHUP, CleanExit);

   /* set a global ptr to the program name so other functions can tell
      what the program name is */
   progname = argv[0];

   InitNetmasks();

   /* Tell 'em who wrote it, and what "it" is */
   DisplayBanner();

   /* initialize the packet counter to loop forever */
   pv.pkt_cnt = -1;

   /* set the timezone (ripped from tcpdump) */
   thiszone = gmt2local(0);

   /* chew up the command line */
   ParseCmdLine(argc, argv);

   if(!pv.log_flag)
   {
      strncpy(pv.log_dir,DEFAULT_LOG_DIR,strlen(DEFAULT_LOG_DIR)+1);  
   }

   /* perform some sanity checks */
   SanityChecks();
 
   if(pv.binfrag_flag)
   {
#ifdef DEBUG
      printf("Initializing binfrag file\n");
#endif
      InitBinFrag();
   }

   if(pv.use_rules && pv.rules_order_flag)
   {
      printf("Rule application order changed to Pass->Alert->Log\n");
   }

   if(!pv.use_rules && !pv.verbose_flag && !pv.log_flag)
   {
      printf("\n\nUh, you need to tell me to do something....\n\n");
      ShowUsage(progname);
      exit(0);
   }

   if(!pv.readmode_flag)
   {
      /* open up our libpcap packet capture interface */
      OpenPcap(pv.interface);

      /* set the packet processor (ethernet, slip or raw)*/
      SetPktProcessor();

      if(pv.daemon_flag) GoDaemon();

      /* Read all packets on the device.  Continue until cnt packets read */
      if(pcap_loop(pd, pv.pkt_cnt, grinder, NULL) < 0)
      {
         fprintf(stderr, "pcap_loop: %s", pcap_geterr(pd));
         CleanExit();
      }

      /* close the capture interface */
      pcap_close(pd);
   }
   else
   {
      ReadPacketsFromFile();
   }

  return 0;
}



/****************************************************************************
 *
 * Function: ShowUsage(char *)
 *
 * Purpose:  Display the program options and exit
 *
 * Arguments: progname => name of the program (argv[0])
 *
 * Returns: 0 => success
 *
 ****************************************************************************/
int ShowUsage(char *progname)
{
   printf("\nUSAGE: %s [-options] <filter options>\n", progname);
   puts("Options:");
   puts("        -a         Display ARP packets");
   puts("        -c <rules> Use Rules File <rules>");
   puts("        -d         Dump the Application Layer");
   puts("        -D         Run Snort in background (daemon) mode");
   puts("        -e         Display the packet Ethernet addresses");
   puts("        -f         Log fragments in binary (tcpdump) format for post-processing");
   puts("        -h <hn>    Home network = <hn>");
   puts("        -i <if>    Listen on interface <if>");
   puts("        -l <ld>    Log to directory <ld>");
   puts("        -M <wrkst> Sends SMB message to workstations in file <wrkst>");
   puts("                   (Requires Samba to be in program PATH)");
   puts("        -n <cnt>   Exit after receiving <cnt> packets");
   puts("        -o         Change the rule testing order to Pass|Alert|Log");
   puts("        -r <tf>    Read and process tcpdump file <tf>");
   puts("        -s         Log alert messages to syslog");
   puts("        -v         Be verbose");
   puts("        -V         Show version number");
   puts("        -x         Display IPX packets");
   puts("        -?         Show this information");
   puts("<Filter Options> are standard BPF options, as seen in TCPDump");
   putc('\n', stdout);

   fflush(stdout);

   return 0;
}




/****************************************************************************
 *
 * Function: ParseCmdLine(int, char *)
 *
 * Purpose:  Parse command line args
 *
 * Arguments: argc => count of arguments passed to the routine
 *            argv => 2-D character array, contains list of command line args
 *
 * Returns: 0 => success, 1 => exit on error
 *
 ****************************************************************************/
int ParseCmdLine(int argc, char *argv[])
{
   char ch;                      /* storage var for getopt info */
   extern char *optarg;          /* for getopt */
   extern int optind;            /* for getopt */

#ifdef DEBUG
   printf("Parsing command line...\n");
#endif

   /* loop through each command line var and process it */
   while((ch = getopt(argc, argv, "DtM:fr:xeh:l:dc:n:i:vV?aso")) != EOF)
   {
#ifdef DEBUG
      printf("Processing cmd line switch: %c\n", ch);
#endif
      switch(ch)
      {
         case 'D': /* daemon mode */
                 pv.daemon_flag = 1;
                 break;

         case 'l': /* use log dir <X> */
                 strncpy(pv.log_dir, optarg, STD_BUF-1);
#ifdef DEBUG
                 printf("Log directory = %s\n", pv.log_dir);
#endif
                 pv.log_flag = 1;
                 break;
                              
         case 'e': /* show Ethernet Header info */
#ifdef DEBUG
                 printf("Show ETH active\n");
#endif
                 pv.showeth_flag = 1;
                 
                 break;

         case 'f': /* log frags in binary format for post-processing */
#ifdef DEBUG
                 printf("BIN-FRAG mode active\n");
#endif

                 pv.binfrag_flag = 1;

                 break;

         case 'a': /* show ARP packets */
#ifdef DEBUG
                 printf("Show ARP active\n");
#endif
                 pv.showarp_flag = 1;
                 
                 break;

         case 'd': /* dump the application layer data */
                 pv.data_flag = 1;
#ifdef DEBUG
                 printf("Data Flag active\n");
#endif
                 break;

         case 'v': /* be verbose */
                 pv.verbose_flag = 1;
#ifdef DEBUG
                 printf("Verbose Flag active\n");
#endif
                 break;

         case 'n': /* grab x packets and exit */
                 pv.pkt_cnt = atoi(optarg);
#ifdef DEBUG
                 printf("Exiting after %d packets\n", pv.pkt_cnt);
#endif
                 break;

         case 'c': /* use configuration file x ( which currently isn't used) */
                 strncpy(pv.config_file, optarg, STD_BUF - 1);
                 pv.use_rules = 1;
                 ParseRulesFile(pv.config_file);
#ifdef DEBUG
                 printf("Config file = %s\n", pv.config_file);
#endif
                 break;

         case 'i': /* listen on interface x */
                 pv.interface = (char *) malloc(strlen(optarg) + 1);
                 bzero(pv.interface, strlen(optarg)+1);
                 strncpy(pv.interface, optarg, strlen(optarg));
#ifdef DEBUG
                 printf("Interface = %s\n", pv.interface);
#endif
                 break;

         case 'o': /* change the rules processing order to passlist first */
		 pv.rules_order_flag = 1;
#ifdef DEBUG
		 printf("Rule application order changed to Pass->Alert->Log\n");
#endif

		 break;

         case 'r': /* read packets from a TCPdump file instead of the net */
                 strncpy(pv.readfile, optarg, STD_BUF - 1);
                 pv.readmode_flag = 1;

                 break;

         case 's': /* log alerts to syslog */
		 pv.syslog_flag = 1;
#ifdef DEBUG
		 printf("Logging alerts to syslog\n");
#endif

		 break;

         case 'x': /* display IPX packets (decoder not implemented yet)*/
#ifdef DEBUG
                 printf("Show IPX active\n");
#endif
                 pv.showipx_flag = 1;
                                 
                 break;

         case 'M': /* SMB Message Option */

                 pv.smbmsg_flag = 1;
                 strncpy(pv.smbmsg_dir, optarg, STD_BUF-1);

                 break;

         case '?': /* show help and exit */
                 ShowUsage(progname);
                 exit(0);

         case 'V': /* prog ver already gets printed out, so we just exit */
                 exit(0);

         case 'h': /* set home network to x, this will help determine what to
                      set logging diectories to */

                 GenHomenet(optarg);

                 break;
      }
   }

   /* set the BPF rules string (thanks Mike!) */
   pv.pcap_cmd = copy_argv(&argv[optind]);

#ifdef DEBUG
   if(pv.pcap_cmd != NULL)
   {
      printf("pcap_cmd = %s\n", pv.pcap_cmd);
   }
   else
   {
      printf("pcap_cmd is NULL!\n");
   }
#endif

   return 0;
}



void GenHomenet(char *netdata)
{
   struct in_addr net;           /* place to stick the local network data */
   char **toks;                  /* dbl ptr to store mSplit return data in */
   int num_toks;                 /* number of tokens mSplit returns */
   int nmask;                    /* temporary netmask storage */

   /* break out the CIDR notation from the IP address */
   toks = mSplit(optarg,"/",2,&num_toks,0);

   if(num_toks > 1)
   {
      /* convert the CIDR notation into a real live netmask */
      nmask = atoi(toks[1]);

      if((nmask > 0) && (nmask < 33))
      {
         pv.netmask = netmasks[nmask];
      }
      else
      {
         fprintf(stderr, "ERROR: Bad CIDR block [%s:%d], 1 to 32 please!\n",
                 toks[1],nmask);
         exit(1);
      }
   }
   else
   {
      fprintf(stderr, "ERROR: No netmask specified for home network!\n");
      exit(1);
   }

   /* since PC's store things the "wrong" way, shuffle the bytes into 
      the right order */
#ifndef WORDS_BIGENDIAN
   pv.netmask = htonl(pv.netmask);
#endif

#ifdef DEBUG
   printf("homenet netmask = %#8lX\n", pv.netmask);
#endif
   /* convert the IP addr into its 32-bit value */
   if((net.s_addr = inet_addr(toks[0])) ==-1)
   {
      fprintf(stderr, "ERROR: Homenet (%s) didn't x-late, WTF?\n",
              toks[0]);
      exit(0);
   }
   else
   {
#ifdef DEBUG
      struct in_addr sin;
      printf("Net = %s (%X)\n", inet_ntoa(net), net.s_addr);
#endif
      /* set the final homenet address up */
      pv.homenet = ((u_long)net.s_addr & pv.netmask);
#ifdef DEBUG
      sin.s_addr = pv.homenet;
      printf("Homenet = %s (%X)\n", inet_ntoa(sin), sin.s_addr);
#endif
   }
 
   free(toks);
}



/****************************************************************************
 *
 * Function: SetPktProcessor()
 *
 * Purpose:  Set which packet processing function we're going to use based on 
 *           what type of datalink layer we're using
 *
 * Arguments: None.
 *
 * Returns: 0 => success
 *
 ****************************************************************************/
int SetPktProcessor()
{
   switch(datalink)
   {
      case DLT_EN10MB:
                printf("Decoding Ethernet on interface %s\n", pv.interface);
                grinder = (pcap_handler) DecodeEthPkt;
                break;

      case DLT_SLIP:
                printf("Decoding Slip on interface %s\n", pv.interface);

		if(pv.showeth_flag == 1)
	        {
		   printf("Disabling Ethernet header printout (you aren't using Ethernet!\n");
		   pv.showeth_flag = 0;
		}

                grinder = (pcap_handler) DecodeSlipPkt;

                break;

#ifdef DLT_RAW /* Not supported in some arch or older pcap versions */
      case DLT_RAW:
                printf("Decoding raw data on interface %s\n", pv.interface);

		if(pv.showeth_flag == 1)
	        {
		   printf("Disabling Ethernet header printout (you aren't using Ethernet!\n");
		   pv.showeth_flag = 0;
		}

                grinder = (pcap_handler) DecodeRawPkt;

                break;
#endif

       default:
                fprintf(stderr, "\n%s cannot handle data link type %d", 
                        progname, datalink);
                CleanExit();
    }
 
   return 0;
}
   

/****************************************************************************
 *
 * Function: OpenPcap(char *)
 *
 * Purpose:  Open the libpcap interface
 *
 * Arguments: intf => name of the interface to open 
 *
 * Returns: 0 => success, exits on problems
 *
 ****************************************************************************/
int OpenPcap(char *intf)
{
   bpf_u_int32 localnet, netmask;    /* net addr holders */
   struct bpf_program fcode;         /* Finite state machine holder */
   char errorbuf[PCAP_ERRBUF_SIZE];  /* buffer to put error strings in */
 
   /* look up the device and get the handle */
   if(pv.interface == NULL)
   {
      pv.interface = pcap_lookupdev(errorbuf);

      if(pv.interface == NULL)
      {
         fprintf(stderr, "ERROR: OpenPcap() interface lookup: \n\t%s\n", 
                 errorbuf);
         exit(1);
      }
   }
 
   /* get the device file descriptor */
   pd = pcap_open_live(pv.interface, SNAPLEN, PROMISC, READ_TIMEOUT, errorbuf);

   if (pd == NULL) 
   {
      fprintf(stderr, "ERROR: OpenPcap() device %s open: \n\t%s\n", 
              pv.interface, errorbuf);
      exit(1);
   }
 
   /* get local net and netmask */
   if(pcap_lookupnet(pv.interface, &localnet, &netmask, errorbuf) < 0)
   {
      fprintf(stderr, "ERROR: OpenPcap() device %s network lookup: \n\t%s\n", 
              pv.interface, errorbuf);
      exit(1);
   }
  
   /* compile command line filter spec info fcode FSM */
   if(pcap_compile(pd, &fcode, pv.pcap_cmd, 0, netmask) < 0)
   {
      fprintf(stderr, "ERROR: OpenPcap() FSM compilation failed: \n\t%s\n", 
              pcap_geterr(pd));
      exit(1);
   } 
  
   /* set the pcap filter */
   if(pcap_setfilter(pd, &fcode) < 0)
   {
      fprintf(stderr, "ERROR: OpenPcap() setfilter: \n\t%s\n", pcap_geterr(pd));
      exit(1);
   }
 
   /* get data link type */
   datalink = pcap_datalink(pd);

   if (datalink < 0) 
   {
      fprintf(stderr, "ERROR: OpenPcap() datalink grab: \n\t%s\n", pcap_geterr(pd));
      exit(1);
   }

   return 0;
}
 
/****************************************************************************
 *
 * Function: CleanExit()
 *
 * Purpose:  Clean up misc file handles and such and exit
 *
 * Arguments: None.
 *
 * Returns: void function
 *
 ****************************************************************************/
void CleanExit()
{
   struct pcap_stat ps;
   float drop;
   float recv;


   /* make sure everything that needs to go to the screen gets there */
   fflush(stdout);

   printf("\nExiting...\n");

   if(pv.readmode_flag)
   {
      exit(0);
   }

   if(pv.binfrag_flag)
   {
      fclose(binfrag_ptr);
   }

   if(pv.smbmsg_flag)
   {
      unlink("/tmp/.snortmsg");
   }

   /* collect the packet stats */
   if(pcap_stats(pd, &ps))
   {
      pcap_perror(pd, "pcap_stats");
   }
   else
   {
      recv = ps.ps_recv;
      drop = ps.ps_drop;

      puts("\n\n===============================================================================");
      printf("Snort received %d packets", ps.ps_recv);

      if(ps.ps_recv)
      {
#ifndef LINUX
         printf(" and dropped %d(%.3f%%) packets\n\n", ps.ps_drop, 
                CalcPct(drop, recv));
#else
         printf(".\nPacket loss statistics are unavailable under Linux.  Sorry!\n\n");
#endif
      }
      else
      {
         puts(".\n");
      }
      puts("Breakdown by protocol:");
      printf("    TCP: %-10ld (%.3f%%)\n", pc.tcp, CalcPct((float)pc.tcp, recv));
      printf("    UDP: %-10ld (%.3f%%)\n", pc.udp, CalcPct((float)pc.udp, recv));
      printf("   ICMP: %-10ld (%.3f%%)\n", pc.icmp, CalcPct((float)pc.icmp, recv));
      printf("    ARP: %-10ld (%.3f%%)\n", pc.arp, CalcPct((float)pc.arp, recv));
      printf("    IPX: %-10ld (%.3f%%)\n", pc.ipx, CalcPct((float)pc.ipx, recv));
      printf("  OTHER: %-10ld (%.3f%%)\n", pc.other, CalcPct((float)pc.other, recv));

      puts("===============================================================================");
   }


   pcap_close(pd);

   exit(0);
}


float CalcPct(float cnt, float total)
{
   float pct;

   if(cnt > 0.0)
      pct = cnt/total;
   else
      return 0.0;

   pct *= 100.0;

   return pct;
}


/****************************************************************************
 *
 * Function: DisplayBanner()
 *
 * Purpose:  Show valuable proggie info
 *
 * Arguments: None.
 *
 * Returns: 0 all the time
 *
 ****************************************************************************/
int DisplayBanner()
{
   printf("\n-*> Snort! <*-\nVersion %s, By Martin Roesch (roesch@clark.net)\n", VERSION);
   return 0;
}



/****************************************************************************
 *  
 * Function: ts_print(register const struct, char *)
 * 
 * Purpose: Generate a time stamp and stuff it in a buffer.  This one has
 *          millisecond precision.  Oh yeah, I ripped this code off from
 *          TCPdump, props to those guys.
 * 
 * Arguments: timeval => clock struct coming out of libpcap
 *            timebuf => buffer to stuff timestamp into
 *      
 * Returns: void function
 * 
 ****************************************************************************/
void ts_print(register const struct timeval *tvp, char *timebuf)
{               
   register int s;  
   struct tm *lt;   /* place to stick the adjusted clock data */

   lt = localtime((time_t *)&tvp->tv_sec);

   s = (tvp->tv_sec + thiszone) % 86400;

   (void)sprintf(timebuf, "%02d/%02d-%02d:%02d:%02d.%06u ", lt->tm_mon+1, 
                 lt->tm_mday, s / 3600, (s % 3600) / 60, s % 60, 
                 (u_int)tvp->tv_usec);
}



/****************************************************************************
 *  
 * Function: gmt2local(time_t)
 * 
 * Purpose: Figures out how to adjust the current clock reading based on the
 *          timezone you're in.  Ripped off from TCPdump.
 * 
 * Arguments: time_t => offset from GMT
 *      
 * Returns: offset seconds from GMT
 * 
 ****************************************************************************/
int gmt2local(time_t t)
{
   register int dt, dir;
   register struct tm *gmt, *loc;
   struct tm sgmt;
 
   if(t == 0)
      t = time(NULL);

   gmt = &sgmt;
   *gmt = *gmtime(&t);
   loc = localtime(&t);

   dt = (loc->tm_hour - gmt->tm_hour) * 60 * 60 + 
        (loc->tm_min - gmt->tm_min) * 60;

   /* If the year or julian day is different, we span 00:00 GMT
     and must add or subtract a day. Check the year first to
     avoid problems when the julian day wraps. */

   dir = loc->tm_year - gmt->tm_year;

   if(dir == 0)
      dir = loc->tm_yday - gmt->tm_yday;

   dt += dir * 24 * 60 * 60;

   return (dt);
}




/****************************************************************************
 *
 * Function: copy_argv(u_char **)
 *
 * Purpose: Copies a 2D array (like argv) into a flat string.  Stolen from
 *          TCPDump.
 *
 * Arguments: argv => 2D array to flatten
 *
 * Returns: Pointer to the flat string
 *
 ****************************************************************************/
char *copy_argv(char **argv)
{
  char **p;
  u_int len = 0;
  char *buf;
  char *src, *dst;
  void ftlerr(char *, ...);

  p = argv;
  if (*p == 0) return 0;

  while (*p)
    len += strlen(*p++) + 1;

  buf = (char *) malloc (len);
  if(buf == NULL)
  {
     fprintf(stderr, "malloc() failed: %s\n", strerror(errno));
     exit(0);
  }
  p = argv;
  dst = buf;
  while ((src = *p++) != NULL)
    {
      while ((*dst++ = *src++) != '\0');
      dst[-1] = ' ';
    }
  dst[-1] = '\0';

  return buf;
}




/****************************************************************************
 *
 * Function: strip(char *)
 *
 * Purpose: Strips a data buffer of CR/LF/TABs.  Replaces CR/LF's with
 *          NULL and TABs with spaces.
 *
 * Arguments: data => ptr to the data buf to be stripped
 *
 * Returns: size of the newly stripped string
 *
 ****************************************************************************/
int strip(char *data)
{
   int size;
   char *end;
   char *idx;

   idx = data;
   end = data + strlen(data);
   size = end - idx;

   while(idx != end)
   {
      if((*idx == '\n') ||
         (*idx == '\r'))
      {
         *idx = 0;
         size--;
      }

      if(*idx == '\t')
      {
         *idx = ' ';
      }

      idx++;
   }

   return size;
}




void ReadPacketsFromFile()
{
   pcap_file_header pfh;
   pcap_pkthdr pph;
   u_char packet[1501];
   FILE *fp = NULL;

#ifdef DEBUG
   printf("Opening %s for reading, fp = %p\n", pv.readfile, fp);
#endif

   if((fp = fopen(pv.readfile, "r")) == NULL)
   {
      perror("OpenPcapFile() fopen");
      exit(1);
   }

   ReadPcapHeader(fp, &pfh);

   printf("\nPcap file header =>\n");
   printf("Magic Number: %X  version: %d.%d  snaplen: %d  linktype: %d\n", 
          pfh.magic, pfh.version_major, pfh.version_minor, pfh.snaplen, 
          pfh.linktype);

   bzero(packet, 1501);

   while(GetNextPkt(fp, &pph, packet))
   {
      DecodeEthPkt(NULL, (struct pcap_pkthdr *)&pph, packet);
      bzero(packet, 1501);
   }

   fclose(fp);
}




void InitNetmasks()
{
   netmasks[0] = 0x0;
   netmasks[1] = 0x80000000;
   netmasks[2] = 0xC0000000;
   netmasks[3] = 0xE0000000;
   netmasks[4] = 0xF0000000;
   netmasks[5] = 0xF8000000;
   netmasks[6] = 0xFC000000;
   netmasks[7] = 0xFE000000;
   netmasks[8] = 0xFF000000;
   netmasks[9] = 0xFF800000;
   netmasks[10] = 0xFFC00000;
   netmasks[11] = 0xFFE00000;
   netmasks[12] = 0xFFF00000;
   netmasks[13] = 0xFFF80000;
   netmasks[14] = 0xFFFC0000;
   netmasks[15] = 0xFFFE0000;
   netmasks[16] = 0xFFFF0000;
   netmasks[17] = 0xFFFF8000;
   netmasks[18] = 0xFFFFC000;
   netmasks[19] = 0xFFFFE000;
   netmasks[20] = 0xFFFFF000;
   netmasks[21] = 0xFFFFF800;
   netmasks[22] = 0xFFFFFC00;
   netmasks[23] = 0xFFFFFE00;
   netmasks[24] = 0xFFFFFF00;
   netmasks[25] = 0xFFFFFF80;
   netmasks[26] = 0xFFFFFFC0;
   netmasks[27] = 0xFFFFFFE0;
   netmasks[28] = 0xFFFFFFF0;
   netmasks[29] = 0xFFFFFFF8;
   netmasks[30] = 0xFFFFFFFC;
   netmasks[31] = 0xFFFFFFFE;
   netmasks[32] = 0xFFFFFFFF;
}




void InitBinFrag()
{
   struct pcap_file_header pfh;
   char logdir[STD_BUF];

   bzero(logdir, STD_BUF);

   sprintf(logdir,"%s/binfrag", pv.log_dir);

#ifdef DEBUG
   printf("Opening %s\n", logdir);
#endif

   /* overwrites the last frag file */
   if((binfrag_ptr = fopen(logdir, "w")) == NULL)
   {
      perror("InitBinFrag()");
      exit(1);
   }

   /* set to tcpdump defaults */
   pfh.magic = 0x3c4d1a2b;
   pfh.version_major = 2;
   pfh.version_minor = 4;
   pfh.thiszone = 0;
   pfh.sigfigs = 6;
   pfh.snaplen = 1500;
   pfh.linktype = datalink;

   /* write out the file header */
   fwrite(&pfh, sizeof(struct pcap_file_header), 1, binfrag_ptr);

#ifdef DEBUG
   printf("Binfrag log file initialized\n");
#endif

   return;
}


void GoDaemon(void) 
{
   int fs;

   printf("Initializing daemon mode\n");

   if(getppid() != 1) 
   {
      fs=fork();

      if(fs > 0) 
         exit(0); // parent 

      if(fs < 0)
      {
         perror("fork");
         exit(1);
      }

      setsid();
   }

   // redirect stdin/stdout/stderr to /dev/null 
   close(0);
   close(1);
   close(2);

#ifdef DEBUG
   open("/tmp/snort.debug", O_CREAT|O_RDWR);
#else
   open("/dev/null",O_RDWR);
#endif

   dup(0);
   dup(0);
   umask(0);

   return;
}



/*
 *  basically we only check if logdir exist and writable, since it 
 *  might screw the whole thing in the middle. Any other checks could be
 *  performed here as well.
 * 
 */

void SanityChecks(void)
{
   struct stat st;

   stat(pv.log_dir,&st);

   if(!S_ISDIR(st.st_mode) || access(pv.log_dir,W_OK) == -1) 
   {
      fprintf(stderr,"\n*Error* :"
              "Can not get write to logging directory %s.\n"
              "(directory doesn't "
              "exist or permissions are set incorrectly)\n\n",
              pv.log_dir);
      exit(0);
   }
}

